#!/usr/bin/env python3

import os
import sys
import json
import tqdm
import re
import faiss
import click
import numpy as np
from typing import Union

from reasoningchain.doc_index import DocIndex
from reasoningchain.api import closeai

@click.command()
@click.option('--query', '-q', 'query', type=str, required=True)
@click.option('--index-path', '-p', 'index_path', type=str, required=True)
def main(query:str, index_path:str):
    doc_index = DocIndex()
    print(f"Loading index \x1b[32m{index_path}\x1b[0m ...")
    doc_index.load(index_path)
    print(f"Searching [\x1b[32m{query}\x1b[0m] ...")
    res = doc_index.search(query, 12)
    if not res:
        print("Nothing found", file=sys.stderr)
        return

    segs = []
    for item in sorted(res, key=lambda x:x['offset']):
        if len(segs) > 0 and segs[-1] == item['paragraph']:
            continue
        segs.append(item['paragraph'])

    episodes = '\n'.join([f'{i+1}) {v}' for i, v in enumerate(segs)])

    prompt = '问题相关剧情信息:{\n%s\n}\n\n问题: %s\n尽可能回答: ' % (episodes, query)

    print(f"prompt(length:{len(prompt)}):\n{prompt}")
    print()
    print("Requesting ChatGPT ...")
    res = closeai.chat(prompt, max_tokens=500)
    print(json.dumps(res, indent=4, ensure_ascii=False))

if __name__ == '__main__':
    main()




#!/usr/bin/env python3

import os
import sys
import json
import tqdm
import re
import faiss
import click
import numpy as np
from typing import Union

from reasoningchain.api import closeai

class DocIndex:
    def __init__(self, embedding_engine='CLOSEAI'):
        self._sentences = None
        self._index = None
        self._enbedding_engine = embedding_engine

    def load(self, index_path):
        self._index = faiss.read_index(index_path + '.index')
        with open(index_path + '.sentences') as f:
            self._sentences = json.load(f)
        return self._index is not None and self._sentences is not None

    def save(self, index_path):
        with open(index_path + '.sentences', 'w') as f:
            f.write(json.dumps(self._sentences, ensure_ascii=False))
        return faiss.write_index(self._index, index_path + '.index')

    def _get_embedding(self, text):
        return closeai.get_embedding_with_cache(text)

    def build(self, doc:str, index_type:str='IVFFlat') -> bool:
        sentences = split_to_sentences(doc)
        dim = 0
        embs = []
        for item in tqdm.tqdm(sentences):
            emb = self._get_embedding(item['text'])
            if not emb:
                print(f"Failed to get embedding for sentence:{item['text']}", file=sys.stderr)
                continue

            if not dim:
                dim = len(emb)

            embs.append(emb)
        embs = np.array(embs)

        #index = faiss.index_factory(dim, index_type)
        quantizer = faiss.IndexFlatL2(dim)
        index = faiss.IndexIVFFlat(quantizer, dim, len(sentences), faiss.METRIC_L2)
        index.train(embs)
        index.add(embs)
        self._index = index
        self._sentences = sentences
        return True

    def search(self, query, topk=10):
        query = query.strip()
        if not query:
            return []
        emb = self._get_embedding(query)
        if not emb:
            raise RuntimeError("Failed to get embedding for " + query)
        emb = np.array([emb])
        self._index.nprobe = topk
        D, I = self._index.search(emb, topk)
        res = []
        for i in range(len(I[0])):
            idx = int(I[0][i])
            if idx < 0:
                continue
            sent = self._sentences[idx]
            res.append({
                'sentence': sent,
                'distance': float(D[0][i]),
            })
        return res

    def get_by_index(self, index):
        if index >= 0 and index < len(self._sentences):
            return self._sentences[index]
        return None

def split_to_sentences(text:str):
    sep = set(['\n', '. ', '! ', '? ', '。', '！', '？', '; ', '；'])
    start = 0
    i = 0
    docid = 1
    res = [] # 腾出0号位置
    last_para = [0, -1]
    while i < len(text):
        if text[i] not in sep: 
            i += 1
            continue
        para_start = -1
        while i < len(text) and text[i] in sep:
            if text[i] == '\n':
                if last_para[1] == -1:
                    last_para[1] = i
                para_start = i + 1
            i += 1
        sentence = text[start:i].strip()
        if i > 0 and text[i-1] == '\n':
            para_offset = i-1
        if len(sentence) > 2:
            res.append({
                'text':sentence,
                'offset':start,
                'paragraph': last_para,
            })
            docid += 1
        start = i
        if para_start >= 0:
            last_para = [para_start, -1]
    if start < len(text):
        last_para[1] = len(text)
        sentence = text[start:].strip()
        if len(sentence) > 2:
            res.append({
                'text':sentence,
                'offset':start,
                'paragraph': last_para,
            })
    return res

@click.command()
@click.argument("mode")
@click.option('--doc', '-d', 'doc_file_path', type=str, required=True)
@click.option('--index-type', '-t', 'index_type', type=str, default='FlatL2', required=False)
@click.option('--index-path', '-p', 'index_path', type=str, required=True)
def build_index(mode:str, doc_file_path:str, index_type:str, index_path:str):
    doc_index = DocIndex()
    with open(doc_file_path) as f:
        text = f.read()
        print("Building index...")
        doc_index.build(text)
        doc_index.save(index_path)
    print(f"Index has been saved to \x1b[32m{index_path}\x1b[0m")

@click.command()
@click.argument("mode")
@click.option('--query', '-q', 'query', type=str, required=True)
@click.option('--index-path', '-p', 'index_path', type=str, required=True)
def search_index(mode:str, query:str, index_path:str):
    doc_index = DocIndex()
    print(f"Loading index \x1b[32m{index_path}\x1b[0m ...")
    doc_index.load(index_path)
    print(f"Searching [\x1b[32m{query}\x1b[0m] ...")
    res = doc_index.search(query)
    if res:
        print('Results:')
        print(json.dumps(res, indent=4, ensure_ascii=False))
    else:
        print("Nothing found", file=sys.stderr)

if __name__ == '__main__':
    mode = sys.argv[1]
    if mode.startswith('-'):
        mode = 'search'
    if mode == 'build':
        build_index()
    elif mode == 'search':
        search_index()
    sys.exit()
    query = sys.argv[1] if len(sys.argv) > 1 else None

    index_path = "./chatgpt_news.index"
    doc_index = DocIndex()
    if os.path.isfile(index_path + '.doc'):
        doc_index.load(index_path)
    else:
        print("Reading text...")
        text = ''
        for line in sys.stdin:
            text += line

        print(text)
        #sentences = split_to_sentences(text)
        #print(json.dumps(sentences, indent=4, ensure_ascii=False))

        doc_index.build(text)
        doc_index.save(index_path)

    if query:
        res = doc_index.search(query)
        print(json.dumps(res, indent=4, ensure_ascii=False))




#!/usr/bin/env python3
# -*- coding:utf-8 -*-
""" A simple disk cache implemention

@Author: Liu Shaofeng(liushaofeng01@baidu.com)
@Date: 2023/03/18 21:45
"""

import os
import sys
import time
import json
import shelve

def disk_cache(cache_path, expire_time=864000): # 默认10天过期
    cache = DiskCache(cache_path, expire_time)
    def decrator(func):
        def wrapper(key, *args, **kwargs):
            ckey = key
            if args or kwargs:
                ckey = str((key, args, kwargs))
            #print(f"disk_cache() ckey = {ckey}")
            val = cache.get(ckey)
            if val:
                return val
            val = func(key, *args, **kwargs)
            if val is not None:
                cache.set(ckey, val)
            return val
        return wrapper
    return decrator

class DiskCache:
    def __init__(self, cache_path, expire_time):
        path = '/'
        for part in os.path.abspath(cache_path).split('/')[1:-1]:
            if not part:
                continue
            path = os.path.join(path, part)
            if not os.path.exists(path):
                os.mkdir(path)

        self._cache_path = cache_path
        self._expire_time = expire_time
        self._db = shelve.open(cache_path)

    def get(self, key):
        key = str(key)
        #print(f"[DiskCache:{self._cache_path}] get({key})", file=sys.stderr)
        if key is None or key not in self._db:
            return None

        ts = time.time()
        cc = self._db[key]
        if ts - cc['create_time'] < self._expire_time:
            return self._db[key]['value']
        return None

    def set(self, key, value):
        key = str(key)
        #print(f"[DiskCache:{self._cache_path}] set({key}, {value})", file=sys.stderr)
        if value is not None:
            self._db[key] = {
                'value':value,
                'create_time':int(time.time()),
            }

    def __exit__(self):
        if self._db is not None:
            self._db.close()
            self._db = None

    def __del__(self):
        if self._db is not None:
            self._db.close()
            self._db = None

if __name__ == '__main__':
    @disk_cache(cache_path="/tmp/tmp_disk_cache.shelve", expire_time=2)
    def search(key:str) -> str:
        print(f"[search] key={key}")
        return f"This is the result for {key}"

    search(sys.argv[1])




#!/usr/bin/env python3
# -*- coding:utf-8 -*-

import os
import sys
import json
import requests
import traceback

SERP_GOOGLE_SEARCH_MAP_URL = 'https://serpapi.com/search.json?api_key=85f7bf8cfdbaa036ea99a228eabb93cfc1a0bdb1736308e8dffe3efa3cb6d6de&engine=google_maps&type=search&q='
SERP_GOOGLE_SEARCH_IMAGE_URL = 'https://serpapi.com/search?api_key=85f7bf8cfdbaa036ea99a228eabb93cfc1a0bdb1736308e8dffe3efa3cb6d6de&device=desktop&engine=google&google_domain=google.com&tbs=images&tbm=isch&q='

def search_map(query):
    query = query.strip()
    if not query:
        return ""
    try:
        url = SERP_GOOGLE_SEARCH_MAP_URL + query
        response = requests.get(url)
        if response.status_code >= 300:
            return None
        res = json.loads(response.content)
        loc = res['place_results']
        return {
            'address': loc['address'],
            'latitude': loc['gps_coordinates']['latitude'],
            'longitude': loc['gps_coordinates']['longitude'],
        }
    except Exception as e:
        traceback.print_exc(file=sys.stderr)
    return None

def search_image(query):
    try:
        query = query.strip()
        if not query:
            return []

        url = SERP_GOOGLE_SEARCH_IMAGE_URL + query
        response = requests.get(url)
        res = json.loads(response.content)
        if response.status_code >= 300:
            return []
        pics = []
        imgs = res['images_results']
        for i in range(min(10, len(imgs))):
            item = imgs[i]
            #print(json.dumps(item,indent=4,ensure_ascii=False))
            pics.append({
                'title':item['title'],
                'thumbnail': item['thumbnail'],
                'original':  item['original'],
                'width': item['original_width'],
                'height': item['original_height'],
            })
        return pics
    except Exception as e:
        traceback.print_exc(file=sys.stderr)
        return []

if __name__ == '__main__':
    query = sys.argv[1]
    res = search_google_image(query)
    print(json.dumps(res, indent=4, ensure_ascii=False))




#!/usr/bin/env python3

import os
import sys
import click
from reasoningchain.custom_tools.base import get_all_tool_names, load_tools

from langchain.callbacks.base import BaseCallbackHandler
from langchain.llms.base import BaseLLM
from langchain.llms import OpenAI, OpenAIChat

def run(
        query:str,
        agent_name:str="zero-shot-react-description",
        tool_names:list=["PythonExecutor"],
        llm:BaseLLM=None,
        tool_callbacks:dict=None,
        chain_callback_handler:BaseCallbackHandler=None,
    ):
    
    query = query.strip()
    if not query:
        return None

    from langchain import agents
    from langchain.callbacks import set_handler

    if llm is None:
        llm = OpenAIChat(temperature=0)

    tools = load_tools(tool_names, tool_callbacks)

    if chain_callback_handler is not None:
        set_handler(chain_callback_handler)

    agent = agents.initialize_agent(tools, llm, agent=agent_name, verbose=True)
    return agent.run(query)

@click.command()
@click.option("--query", "-q", "query", type=str, required=True)
@click.option("--tools", "-t", "tool_names", type=str, default="BaiduSearchText", required=False)
@click.option("--llm",   "-m", "llm_name", type=str, default="OpenAIChat", required=False)
@click.option("--agent", "-g", "agent_name", type=str, default="zero-shot-react-description", required=False)
def cli(query:str, tool_names:str, llm_name:str, agent_name:str) -> int:
    tool_name_list = tool_names.split(",")
    if not llm_name:
        llm_name == 'OpenAIChat'
    if llm_name == 'OpenAIChat':
        llm = OpenAIChat(temperature=0)
    else:
        llm = None
    answer = run(query=query, agent_name=agent_name, tool_names=tool_name_list, llm=llm)

    print(f" Query:\x1b[33m{query}\x1b[0m")
    print(f"Params:tools=[\x1b[33m{tool_names}\x1b[0m]\tllm=[\x1b[33m{llm_name}\x1b[0m]\tagent=[\x1b[33m{agent_name}\x1b[0m]")
    print(f"Answer:\x1b[1;32m{answer}\x1b[0m")
    return 0

if __name__ == '__main__':
    sys.exit(cli())




#!/usr/bin/env python3
# -*- coding:utf-8 -*-

import os
import sys
import json
import requests
import traceback

def search_text(query, max_result_length=600):
    query = query.strip()
    if not query:
        return ""

    try:
        url = os.environ['BAIDU_SEARCH_API'] + query
        response = requests.get(url)
        if response.status_code >= 300:
            return ''
        res = json.loads(response.content)
        res_data = res['data']
        items = res_data['tplData']['asResult']['item']
        res_text = ''
        seq = 1
        res_items = []
        total_length = 0
        for i in range(min(len(items), 5)):
            if total_length >= max_result_length:
                break
            item = items[i]
            result = item.get('result', None)
            if not result:
                continue
            title = ''
            content = ''
            if 'dqa_result' in result:
                title = result['dqa_result'].get('title', '')
                content = result['dqa_result']['content']
            else:
                title = result.get('title', '')
                for k in ('abstraction', 'tts'):
                    content = result.get(k)
                    if content:
                        break
            if title or content:
                text  = f'{seq}) ' + title + '\n'
                text += content + '\n'
                res_items.append(text)
                seq += 1
                total_length += len(text)
        res_text = '\n'.join(res_items)
        if len(res_text) > max_result_length:
            return res_text[:max_result_length-3] + '...'
        return res_text
    except Exception as e:
        traceback.print_exc(file=sys.stderr)
        return ""

if __name__ == '__main__':
    query = sys.argv[1]
    res = search_text(query)
    print(res)

